package de.ctdo.bunti.model;

import java.util.Map;

public abstract class BuntiDevice {
    private int deviceId;
    private String deviceName;
    private long lastChanged;

    public BuntiDevice(int deviceId, String deviceName, long lastChanged) {
        this.deviceId = deviceId;
        this.deviceName = deviceName;
        this.lastChanged = lastChanged;
    }

    public BuntiDevice(int deviceId, String deviceName) {
        this(deviceId, deviceName, System.currentTimeMillis());
    }

    public final int getDeviceId() {
        return deviceId;
    }

    public final long getLastChanged() {
        return lastChanged;
    }

    protected final void lastChangedNow() {
        this.lastChanged = System.currentTimeMillis();
    }

    public final String getDeviceName() {
        return deviceName;
    }

    public final void setDeviceName(String deviceName) {
        this.deviceName = deviceName;
    }

    /**
     * Switch this device off.
     */
    public abstract void switchOff();

    /**
     * Switch this device on.
     */
    public abstract void switchOn();

    /**
     * The the internal options corresponding to the given Key Value Map
     *
     * @param options The options Map.
     * @return True on success. False otherwise.
     */
    public abstract boolean setValuesFromOptions(Map<String, Object> options);

}
